<?php

/**
 * JMultilingualActiveRecord class file.
 *
 * @reference multilingual-active-record <http://www.yiiframework.com/extension/multilingual-active-record/> <created by guillemc>
 * @version alpha3
 * @author jerry2801 <jerry2801@gmail.com>
 *
 * A typical usage of JMultilingualActiveRecord is as follows:
 * <pre>
 * class Product extends JMultilingualActiveRecord
 * {
 *     public function localizedAttributes()
 *     {
 *         return array('name','content');
 *     }
 *
 *     public function languages()
 *     {
 *         return array('zh_cn');
 *     }
 *
 *     public function primaryLang()
 *     {
 *         return array('en_us');
 *     }
 * }
 * </pre>
 */
class JMultilingualActiveRecord extends CActiveRecord
{
  
    protected $_langAttributes = array();
    protected $overwriteIfEmpty = false;

    protected $dropLangAfterDelete = true;

    public function __get($name)
	{
		if(array_key_exists($name, $this->_langAttributes))
			return $this->_langAttributes[$name];
		else
			return parent::__get($name);
	}

	public function __set($name,$value)
	{
		if(array_key_exists($name, $this->_langAttributes))
            $this->_langAttributes[$name] = $value;
        else
            parent::__set($name,$value);
	}
  
	public function __isset($name)
	{
		if(isset($this->_langAttributes[$name]))
			return true;
		else
			return parent::__isset($name);
	}

    public function localizedAttributes()
    {
        return array();
    }

    public function languages()
    {
        return array();
    }

    public function primaryLang()
    {
        return null;
    }

    public function langClassName()
    {
        return $this->tableName().'Mui';
    }

    public function langForeignKey()
    {
        return 'owner_id';
    }

    public function langField()
    {
        return 'lang';
    }
  
  
    public function localized($lang=null, $skipEmptyBy=null)
    {
        if(is_null($lang))
            $lang = Yii::app()->language;
        
        if($lang == $this->primaryLang())
        {
            if(! is_null($skipEmptyBy))
                $this->getDbCriteria()->mergeWith(array('condition'=>$skipEmptyBy.'!=\'\''));
            return $this;
        }

        $class = self::HAS_MANY;
        $options = array('index'=>$this->langField());
        $condition='localized.'.$this->langField().'=\''.$lang.'\'';
        if(! is_null($skipEmptyBy))
            $condition.=' AND localized.'.$skipEmptyBy.'!=\'\'';
        $options['condition']=$condition;
        $this->getMetaData()->relations['localized'] = new $class('localized', $this->langClassName(), $this->langForeignKey(),$options);
        return $this->with('localized')->together();
    }



    public function multilingual()
    {
        $class = self::HAS_MANY;
        $options = array('index'=>$this->langField());
        $this->getMetaData()->relations['multilingual'] = new $class('multilingual', $this->langClassName(), $this->langForeignKey(),$options);
        return $this->with('multilingual');
    }
  
    protected function afterFind()
    {
        if ($this->hasRelated('localized'))
        {
            $fields = $this->localizedAttributes();
            $related = $this->getRelated('localized');
            if ($row = current($related))
            {
                foreach ($fields as $field)
                    if ($row[$field] || $this->overwriteIfEmpty) $this->$field = $row[$field];
            }
        }
        else if ($this->hasRelated('multilingual'))
        {
            $fields = $this->localizedAttributes();
            $related = $this->getRelated('multilingual');
            $languages = $this->languages();
            foreach ($languages as $lang)
                foreach ($fields as $field)
                    $this->_langAttributes[$field.'_'.$lang]=isset($related[$lang][$field])?$related[$lang][$field]:null; 
        }
        if($this->hasEventHandler('onAfterFind'))
            $this->onAfterFind(new CEvent($this));
    }
  
    protected function afterConstruct()
    {
        $class = $this->langClassName();
        $obj = new $class;
        $fields = $this->localizedAttributes();
        $languages = $this->languages();
        foreach ($languages as $lang)
            foreach ($fields as $field)
                $this->_langAttributes[$field.'_'.$lang]=$obj->$field;

        if($this->hasEventHandler('onAfterConstruct'))
            $this->onAfterConstruct(new CEvent($this));
    }
  
    protected function afterSave()
    {
        $class = $this->langClassName();
        $model = call_user_func(array($class, 'model'));
        $languages = $this->languages();
        $langField = $this->langField();
        $foreignKey = $this->langForeignKey();
        $fields = $this->localizedAttributes();
        foreach ($languages as $lang)
        {
            $obj = $model->find("$foreignKey = :id AND $langField = :lang", array('id' => $this->id, ':lang'=>$lang));
            if (! $obj)
            {
                $obj = new $class;
                $obj->$langField = $lang;
                $obj->$foreignKey = $this->id;
            }
            foreach ($fields as $field)
            {
                $f = $field.'_'.$lang;
                $obj->$field = $this->$f;
            }
            $obj->save();
        }
        if($this->hasEventHandler('onAfterSave'))
            $this->onAfterSave(new CEvent($this));
    }
  
  





    public function assignLangAttributes($data)
    {
        foreach($data as $lang => $attributes)
            foreach ($attributes as $field=>$value)
                $this->_langAttributes[$field.'_'.$lang]=$value;
    }

    protected function afterDelete()
    {
        if($this->dropLangAfterDelete)
        {
            $class = $this->langClassName();
            $model = call_user_func(array($class, 'model'));
            $foreignKey = $this->langForeignKey();
            $model->deleteAllByAttributes(array($foreignKey => $this->id));
        }

        return parent::afterDelete();
    }


}
